@@ -1,11 +1,11 @@
-from multiprocessing import Process, Pool
+from multiprocessing import Pool
 import os
 from datetime import datetime
 import copy
 import logging
-import socket
 import traceback
 import random
+import time
 
 import yaml
 import redcap
@@ -18,18 +18,18 @@ from .launcher import BUILD_SUFFIX
 from . import log
 from .errors import AutoProcessorError, DaxError
 from . import utilities
+from . import rcq
+from .XnatUtils import get_interface
 
 # dax manager has 3 main classes: DaxManager has a DaxProjectSettingsManager
 # which is a collection of DaxProjectSettings.
 
 # TODO: archive old logs
 
-# TODO: only run launch and update if there are open jobs
-
 
 DAX_SETTINGS = DAX_Settings()
 
-LOGGER = log.setup_debug_logger('manager', None)
+LOGGER = log.setup_info_logger('manager', None)
 
 
 def project_from_settings(settings_file):
@@ -37,13 +37,6 @@ def project_from_settings(settings_file):
     return proj
 
 
-def get_this_instance():
-    # build the instance name
-    this_host = socket.gethostname().split('.')[0]
-    this_user = os.environ['USER']
-    return '{}@{}'.format(this_user, this_host)
-
-
 def is_locked(settings_path, lock_dir):
     _prefix = os.path.splitext(os.path.basename(settings_path))[0]
     flagfile = os.path.join(lock_dir, '{}_{}'.format(_prefix, BUILD_SUFFIX))
@@ -200,16 +193,30 @@ class DaxProjectSettingsManager(object):
         default_settings = self.load_defaults(DaxProjectSettings())
 
         for name in self.project_names():
+            # Initialize settings to be written to file for this project
             settings = copy.deepcopy(default_settings)
 
             try:
                 LOGGER.info('Loading project: ' + name)
                 project = self.load_project(settings, name)
-                settings.add_project(project)
-
                 filename = os.path.join(
                     self._local_dir, 'settings-{}.yaml'.format(name))
 
+                # Check for empty processors/modules
+                if not project['modules'] and not project['yamlprocessors']:
+                    LOGGER.info('empty processors/modules')
+                    # Delete existing
+                    if os.path.exists(filename):
+                        LOGGER.info(f'deleting stray settings file:{filename}')
+                        os.remove(filename)
+
+                    # Nothing else to do with this project
+                    continue
+
+                # Combine loaded project settings with instance settings
+                settings.add_project(project)
+
+                # Compare with settings in existing file
                 if self.settings_match(settings, filename):
                     LOGGER.info('settings unchanged:{}'.format(filename))
                 else:
@@ -426,7 +433,7 @@ class DaxProjectSettingsManager(object):
         instance_field = 'gen_daxinstance'
 
         # Filter to only include projects for this instance that are Complete
-        this_instance = get_this_instance()
+        this_instance = utilities.get_this_instance()
         plist = [
             r['project_name'] for r in self.records if
             r[instance_field] == this_instance and
@@ -582,7 +589,7 @@ class DaxManager(object):
     FDATEFORMAT = '%Y%m%d-%H%M%S'
     DDATEFORMAT = '%Y%m%d'
 
-    def __init__(self, api_url, api_key_instances, api_key_projects):
+    def __init__(self, api_url, api_key_instances, api_key_projects, api_key_rcq=None):
 
         # TODO: test the api keys or catch errors from pycap to
         # handle when redcap is down
@@ -613,6 +620,11 @@ class DaxManager(object):
             instance_settings,
             self.settings_dir)
 
+        if api_key_rcq:
+            self._rcq = redcap.Project(api_url, api_key_rcq)
+        else:
+            self._rcq = None
+
     def is_enabled_instance(self):
         return (self.enabled)
 
@@ -638,7 +650,7 @@ class DaxManager(object):
         self._redcap = redcap.Project(redcap_url, redcap_key)
 
         # get this instance name
-        instance_name = get_this_instance()
+        instance_name = utilities.get_this_instance()
         LOGGER.debug('instance={}'.format(instance_name))
 
         # Return the record associated with this instance_name
@@ -663,12 +675,12 @@ class DaxManager(object):
         dname = datetime.strftime(timestamp, self.DDATEFORMAT)
         fname = '{}_{}_{}.log'.format(
             runtype, project, datetime.strftime(timestamp, self.FDATEFORMAT))
-        log = os.path.join(self.log_dir, project, dname, fname)
+        _log = os.path.join(self.log_dir, project, dname, fname)
 
         # Make sure the parent dirs exist
-        make_parents(log)
+        make_parents(_log)
 
-        return log
+        return _log
 
     def queue_builds(self, build_pool):
         # TODO: sort builds by how long we expect them to take,
@@ -680,15 +692,15 @@ class DaxManager(object):
         # Run each
         for i, settings_path in enumerate(self.settings_list):
             proj = project_from_settings(settings_path)
-            log = self.log_name('build', proj, datetime.now())
+            _log = self.log_name('build', proj, datetime.now())
             last_run = self.get_last_run(proj)
 
             LOGGER.info('SETTINGS:{}'.format(settings_path))
             LOGGER.info('PROJECT:{}'.format(proj))
-            LOGGER.info('LOG:{}'.format(log))
+            LOGGER.info('LOG:{}'.format(_log))
             LOGGER.info('LASTRUN:' + str(last_run))
             build_results[i] = build_pool.apply_async(
-                self.run_build, [proj, settings_path, log, last_run])
+                self.run_build, [proj, settings_path, _log, last_run])
 
         return build_results
 
@@ -754,8 +766,8 @@ class DaxManager(object):
                         proj = project_from_settings(settings_path)
 
                         LOGGER.info('updating jobs:' + proj)
-                        log = self.log_name('update', proj, datetime.now())
-                        self.run_update(settings_path, log)
+                        _log = self.log_name('update', proj, datetime.now())
+                        self.run_update(settings_path, _log)
                     except (AutoProcessorError, DaxError) as e:
                         err = 'error running update:project={}\n{}'.format(proj, e)
                         LOGGER.error(err)
@@ -772,8 +784,8 @@ class DaxManager(object):
                         proj = project_from_settings(settings_path)
 
                         LOGGER.info('launching jobs:' + proj)
-                        log = self.log_name('launch', proj, datetime.now())
-                        self.run_launch(settings_path, log)
+                        _log = self.log_name('launch', proj, datetime.now())
+                        self.run_launch(settings_path, _log)
                     except (AutoProcessorError, DaxError) as e:
                         err = 'error running launch:project={}\n{}'.format(proj, e)
                         LOGGER.error(err)
@@ -797,6 +809,64 @@ class DaxManager(object):
                     upload_results = self.queue_uploads(upload_pool)
                     upload_pool.close()
 
+            if self._rcq:
+                _projects = self.settings_manager.project_names()
+                LOGGER.info(f'rcq update:{_projects}')
+                rcq.update(
+                    self._rcq,
+                    self._redcap,
+                    build_enabled=False, 
+                    launch_enabled=self.is_enabled_launch(),
+                    projects=_projects)
+
+                # Run separate rcq build so it locks between dax_managers
+                if self.is_enabled_build():
+                    build_errors = self.run_rcq_build(_projects)
+                    run_errors.extend(build_errors)
+
+            # While builds and uploads are running, wait, run
+            # another rcq update and repeat for max iterations.
+            # Also, each iteration, get updates for build/upload threads.
+            rcq_count = 0
+            max_count = 3333
+            timeout = 1
+            while rcq_count < max_count:
+                time.sleep(180)
+                LOGGER.info(f'rcq update:{rcq_count}')
+                self.rcq_update()
+                rcq_count += 1
+
+                # Check results
+                build_done = all([x.ready() for x in build_results])
+                upload_done = all([x.ready() for x in upload_results])
+                if build_done and upload_done:
+                    LOGGER.info('all builds and uploads done')
+                    break
+
+                # Show current results
+                LOGGER.info('build threads:')
+                for i, r in enumerate(build_results):
+                    LOGGER.info(f'{i}:{r.ready()}')
+                    try:
+                        LOGGER.info(f'{i}:result={r.get(timeout=timeout)}')
+                    except:
+                        LOGGER.info(f'{i}:{timeout}')
+
+                for p in build_pool._pool:
+                    LOGGER.info(f'pid:{p.pid}:{p.is_alive()}:exit={p.exitcode}')
+
+                LOGGER.info('upload threads:')
+                for i, r in enumerate(upload_results):
+                    LOGGER.info(f'{i}:{r.ready()}')
+                    try:
+                        LOGGER.info(f'{i}:result={r.get(timeout=timeout)}')
+                    except:
+                        LOGGER.info(f'{i}:timeout:{timeout}')
+
+                for p in upload_pool._pool:
+                    LOGGER.info(f'pid:{p.pid}:{p.is_alive()}:exit={p.exitcode}')
+
+
             if self.is_enabled_build() and num_build_threads > 0:
                 # Wait for builds to finish
                 LOGGER.info('waiting for builds to finish')
@@ -833,6 +903,14 @@ class DaxManager(object):
 
         return run_errors
 
+    def rcq_update(self):
+        self.rcq.update(
+            self._rcq,
+            self._redcap,
+            build_enabled=False, 
+            launch_enabled=self.is_enabled_launch(),
+            projects=self.settings_manager.project_names())
+
     def run_build(self, project, settings_file, log_file, lastrun):
         build_error = None
 
@@ -851,6 +929,7 @@ class DaxManager(object):
                 dax.bin.build(
                     settings_file, log_file, True, proj_lastrun=proj_lastrun)
                 logging.getLogger('dax').handlers = []
+
             except Exception:
                 err = 'error running build:proj={}:err={}'.format(
                     project, traceback.format_exc())
@@ -861,6 +940,99 @@ class DaxManager(object):
             LOGGER.info('run_build:done:{}'.format(project))
             return build_error
 
+    def run_rcq_build(self, projects):
+        build_errors = []
+        lock_file = f'{self.lock_dir}/RCQ_BUILD.pid'
+
+        success = lockfiles.lock_flagfile(lock_file)
+
+        if not success:
+            LOGGER.warn(f'failed to get lock, cannot build rcq:{lock_file}')
+            return []
+
+        instance_settings = rcq._load_instance_settings(self._redcap)
+        yaml_dir = instance_settings['main_processorlib']
+        with get_interface(host=self.xnat_host) as xnat:
+            builder = rcq.TaskBuilder(self._rcq, xnat, yaml_dir)
+            build_projects = [x for x in builder.build_projects() if x in projects]
+            complete_projects = []
+
+            # Store the start time for all projects
+            self.set_last_build_start_multiple(build_projects)
+
+            for project in build_projects:
+                LOGGER.info(f'rcq build:{project}')
+                try:
+                    builder.update(project)
+                    complete_projects.append(project)
+                except Exception as err:
+                    err = f'rcq build:{project}:err={traceback.format_exc()}'
+                    LOGGER.error(err)
+                    build_errors.append(err)
+
+            # Save the complete time for all projects
+            self.set_last_build_complete_multiple(complete_projects)
+
+        LOGGER.debug(f'deleting lock file:{lock_file}')
+        lockfiles.unlock_flagfile(lock_file)
+
+        return build_errors
+
+    def set_last_build_start_multiple(self, projects):
+        records = []
+        last_start = datetime.strftime(
+            datetime.now(),
+            self.settings_manager.RCDATEFORMAT
+        )
+
+        for p in projects:
+            records.append({
+                'project_name': p,
+                'build_laststarttime': last_start,
+                'build_status_complete': '1'}
+            )
+            LOGGER.info(f'set last build start: project={p}, {last_start}')
+
+        try:
+            response = self.settings_manager._redcap.import_records(records)
+            assert 'count' in response
+        except Exception as err:
+            LOGGER.info(err)
+
+    def set_last_build_complete_multiple(self, projects):
+        records = []
+        last_finish = datetime.strftime(
+            datetime.now(),
+            self.settings_manager.RCDATEFORMAT
+        )
+
+        for p in projects:
+            last_start = self.settings_manager.get_last_start_time(p)
+            last_duration = self.settings_manager.duration(
+                last_start,
+                last_finish
+            )
+
+            records.append({
+                'project_name': p,
+                'build_lastcompletestarttime': last_start,
+                'build_lastcompletefinishtime': last_finish,
+                'build_lastcompleteduration': last_duration,
+                'build_status_complete': '2'}
+            )
+
+            LOGGER.info(
+                'set last build: project={}, start={}, finish={}'.format(
+                    p,
+                    last_start,
+                    last_finish))
+
+        try:
+            response = self.settings_manager._redcap.import_records(records)
+            assert 'count' in response
+        except Exception as err:
+            LOGGER.info(err)
+
     def set_last_build_start(self, project):
         self.settings_manager.set_last_build_start(project)
 
